#!/bin/sh
# Copyright (c) 2000-2021 Synology Inc. All rights reserved.

PACKAGE_NAME="SafeAccess"
PACKAGE_TARGET_DIR="/var/packages/${PACKAGE_NAME}/target"
DNSFILTER_IPTABLES_SH="${PACKAGE_TARGET_DIR}/scripts/dnsfilter-iptables.sh"
PACKAGE_BIN_DIR="${PACKAGE_TARGET_DIR}/bin"
SYNOAC_INIT="${PACKAGE_BIN_DIR}/synoac_init.sh"

source "${PACKAGE_TARGET_DIR}/scripts/dnsfilter-util.sh"

old_context=""
new_context=""
old_ifnames=""
new_ifnames=""
removed_ifnames=""

info_log() {
	echo "[$(date +'%Y/%m/%d %H:%M:%S')][info] dnsfilter-topology-change.sh: $1" >> ${LOG_FILE}
}

parse_states ()
{
	if /usr/syno/bin/synobootseq --is-shutdown > /dev/null 2>&1 ; then
		return 1
	fi

	# Modifies global variables
	old_context=$(${JQ} -r ".old.context" "${ARGS_FILE_PATH}")
	new_context=$(${JQ} -r ".new.context" "${ARGS_FILE_PATH}")
	old_ifnames=$(${JQ} -r ".old.networks | keys[]" "${ARGS_FILE_PATH}" | grep "br" | tr '\n' ' ' | sed 's/ *$//')
	new_ifnames=$(${JQ} -r ".new.networks | keys[]" "${ARGS_FILE_PATH}" | grep "br" | tr '\n' ' ' | sed 's/ *$//')
	removed_ifnames=$(get_set_difference "${old_ifnames}" "${new_ifnames}")
}

context_changed ()
{
	if [ "${old_context}" != "${new_context}" ]; then
		return 0
	fi

	return 1
}

interface_changed ()
{
	if [ "${old_ifnames}" != "${new_ifnames}" ]; then
		return 0
	fi

	return 1
}

case $1 in
	--sdk-mod-ver)
		# Print SDK support version
		echo "1.0";
	;;
	--name)
		# Print package name
		echo "url-blocker"
	;;
	--pkg-ver)
		# Print package version
		echo "1.0";
	;;
	--vendor)
		# Print package vendor
		echo "Synology";
	;;
	--pre)
		info_log "--pre  ##### START #####"
		parse_states

		if context_changed ; then
			info_log "--pre context changes from [${old_context}] to [${new_context}]"
			${DNSFILTER_IPTABLES_SH} stop &
			/sbin/stop synosafeaccessfilterd
		elif interface_changed; then
			info_log "--pre interface changes from [${old_ifnames}] to [${new_ifnames}]"
			/sbin/stop synosafeaccessfilterd
		fi
		info_log "--pre  #####  END  #####"
	;;
	--post)
		info_log "--post ##### START #####"
		parse_states

		if context_changed ; then
			${DNSFILTER_IPTABLES_SH} start &
			start_synosafeaccessfilterd_if_required
		elif interface_changed; then
			if [ "router" = "${new_context}" ]; then
				if [ ! -z "${removed_ifnames}" ]; then
					info_log "Removed interfaces under router mode: [${removed_ifnames}]"
					remove_network_profiles_of_interfaces "${removed_ifnames}"
				fi
			fi
			start_synosafeaccessfilterd_if_required
		fi
		info_log "--post #####  END  #####"
	;;
	*)
		echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
	;;
esac
